@@ -32,6 +32,18 @@
 (defun widetag-of (x)
   (widetag-of x))
 
+;;; WIDETAG-OF needs extra code to handle LIST and FUNCTION lowtags. When
+;;; we're only dealing with other pointers (eg. when dispatching on array
+;;; element type), this is going to be faster.
+(declaim (inline %other-pointer-widetag))
+(defun %other-pointer-widetag (x)
+  (sb!sys:sap-ref-8 (int-sap (get-lisp-obj-address x))
+                    #.(ecase sb!c:*backend-byte-order*
+                        (:little-endian
+                         (- sb!vm:other-pointer-lowtag))
+                        (:big-endian
+                         (- (1- sb!vm:n-word-bytes) sb!vm:other-pointer-lowtag)))))
+
 ;;; Return a System-Area-Pointer pointing to the data for the vector
 ;;; X, which must be simple.
 ;;;
@@ -117,15 +129,6 @@
 (defun code-header-set (code-obj index new)
   (code-header-set code-obj index new))
 
-(defun %raw-bits (object offset)
-  (declare (type index offset))
-  (sb!kernel:%raw-bits object offset))
-
-(defun %set-raw-bits (object offset value)
-  (declare (type index offset))
-  (declare (type sb!vm:word value))
-  (setf (sb!kernel:%raw-bits object offset) value))
-
 (defun %vector-raw-bits (object offset)
   (declare (type index offset))
   (sb!kernel:%vector-raw-bits object offset))
