@@ -24,12 +24,12 @@
 (declaim (type readtable *readtable*))
 #!+sb-doc
 (setf (fdocumentation '*readtable* 'variable)
-       "Variable bound to current readtable.")
+      "Variable bound to current readtable.")
 
-;;; a standard Lisp readtable. This is for recovery from broken
-;;; read-tables (and for WITH-STANDARD-IO-SYNTAX), and should not
-;;; normally be user-visible.
-(defvar *standard-readtable*)
+;;; A standard Lisp readtable (once cold-init is through). This is for
+;;; recovery from broken read-tables (and for
+;;; WITH-STANDARD-IO-SYNTAX), and should not normally be user-visible.
+(defvar *standard-readtable* nil)
 
 (defvar *old-package* nil
   #!+sb-doc
@@ -59,33 +59,34 @@
 
 ;;;; macros and functions for character tables
 
-;;; FIXME: could be SB!XC:DEFMACRO inside EVAL-WHEN (COMPILE EVAL)
-(defmacro get-cat-entry (char rt)
-  ;; KLUDGE: Only give this side-effect-free args.
-  ;; FIXME: should probably become inline function
-  `(if (typep ,char 'base-char)
-       (elt (character-attribute-array ,rt) (char-code ,char))
-       (gethash ,char (character-attribute-hash-table ,rt)
-        +char-attr-constituent+)))
+(defun get-cat-entry (char rt)
+  (declare (readtable rt))
+  (if (typep char 'base-char)
+      (elt (character-attribute-array rt) (char-code char))
+      (values (gethash char (character-attribute-hash-table rt)
+                       +char-attr-constituent+))))
 
 (defun set-cat-entry (char newvalue &optional (rt *readtable*))
+  (declare (readtable rt))
   (if (typep char 'base-char)
       (setf (elt (character-attribute-array rt) (char-code char)) newvalue)
-      ;; FIXME: could REMHASH if we're setting to
-      ;; +CHAR-ATTR-CONSTITUENT+
-      (setf (gethash char (character-attribute-hash-table rt)) newvalue)))
+      (if (= newvalue +char-attr-constituent+)
+          ;; Default value for the C-A-HASH-TABLE is +CHAR-ATTR-CONSTITUENT+.
+          (%remhash char (character-attribute-hash-table rt))
+          (setf (gethash char (character-attribute-hash-table rt)) newvalue)))
+  (values))
 
 ;;; the value actually stored in the character macro table. As per
 ;;; ANSI #'GET-MACRO-CHARACTER and #'SET-MACRO-CHARACTER, this can
 ;;; be either a function or NIL.
-(eval-when (:compile-toplevel :execute)
-  (sb!xc:defmacro get-raw-cmt-entry (char readtable)
-    `(if (typep ,char 'base-char)
-         (svref (character-macro-array ,readtable) (char-code ,char))
-         ;; Note: DEFAULT here is NIL, not #'UNDEFINED-MACRO-CHAR, so
-         ;; that everything above the base-char range is a non-macro
-         ;; constituent by default.
-         (gethash ,char (character-macro-hash-table ,readtable) nil))))
+(defun get-raw-cmt-entry (char readtable)
+  (declare (readtable readtable))
+  (if (typep char 'base-char)
+      (svref (character-macro-array readtable) (char-code char))
+      ;; Note: DEFAULT here is NIL, not #'UNDEFINED-MACRO-CHAR, so
+      ;; that everything above the base-char range is a non-macro
+      ;; constituent by default.
+      (values (gethash char (character-macro-hash-table readtable) nil))))
 
 ;;; the value represented by whatever is stored in the character macro
 ;;; table. As per ANSI #'GET-MACRO-CHARACTER and #'SET-MACRO-CHARACTER,
@@ -97,13 +98,11 @@
         #'read-token)))
 
 (defun set-cmt-entry (char new-value-designator &optional (rt *readtable*))
-  (if (typep char 'base-char)
-      (setf (svref (character-macro-array rt) (char-code char))
-            (and new-value-designator
-                 (%coerce-callable-to-fun new-value-designator)))
-      (setf (gethash char (character-macro-hash-table rt))
-        (and new-value-designator
-                 (%coerce-callable-to-fun new-value-designator)))))
+  (let ((new (when new-value-designator
+               (%coerce-callable-to-fun new-value-designator))))
+    (if (typep char 'base-char)
+        (setf (svref (character-macro-array rt) (char-code char)) new)
+        (setf (gethash char (character-macro-hash-table rt)) new))))
 
 (defun undefined-macro-char (stream char)
   (unless *read-suppress*
@@ -116,29 +115,35 @@
 
 ;;; predicates for testing character attributes
 
+#!-sb-fluid
+(progn
+  (declaim (inline whitespace[1]p whitespace[2]p))
+  (declaim (inline constituentp terminating-macrop))
+  (declaim (inline single-escape-p multiple-escape-p))
+  (declaim (inline token-delimiterp)))
+
 ;;; the [1] and [2] here refer to ANSI glossary entries for
 ;;; "whitespace".
-#!-sb-fluid (declaim (inline whitespace[1]p whitespace[2]p))
 (defun whitespace[1]p (char)
   (test-attribute char +char-attr-whitespace+ *standard-readtable*))
 (defun whitespace[2]p (char &optional (rt *readtable*))
   (test-attribute char +char-attr-whitespace+ rt))
 
-(defmacro constituentp (char &optional (rt '*readtable*))
-  `(test-attribute ,char +char-attr-constituent+ ,rt))
+(defun constituentp (char &optional (rt *readtable*))
+  (test-attribute char +char-attr-constituent+ rt))
 
-(defmacro terminating-macrop (char &optional (rt '*readtable*))
-  `(test-attribute ,char +char-attr-terminating-macro+ ,rt))
+(defun terminating-macrop (char &optional (rt *readtable*))
+  (test-attribute char +char-attr-terminating-macro+ rt))
 
-(defmacro single-escape-p (char &optional (rt '*readtable*))
-  `(test-attribute ,char +char-attr-single-escape+ ,rt))
+(defun single-escape-p (char &optional (rt *readtable*))
+  (test-attribute char +char-attr-single-escape+ rt))
 
-(defmacro multiple-escape-p (char &optional (rt '*readtable*))
-  `(test-attribute ,char +char-attr-multiple-escape+ ,rt))
+(defun multiple-escape-p (char &optional (rt *readtable*))
+  (test-attribute char +char-attr-multiple-escape+ rt))
 
-(defmacro token-delimiterp (char &optional (rt '*readtable*))
-  ;; depends on actual attribute numbering above.
-  `(<= (get-cat-entry ,char ,rt) +char-attr-terminating-macro+))
+(defun token-delimiterp (char &optional (rt *readtable*))
+  ;; depends on actual attribute numbering in readtable.lisp.
+  (<= (get-cat-entry char rt) +char-attr-terminating-macro+))
 
 ;;;; constituent traits (see ANSI 2.1.4.2)
 
@@ -181,18 +186,31 @@
                    return-char-code rubout-char-code))
     (!set-constituent-trait (code-char c) +char-attr-invalid+)))
 
-(defmacro get-constituent-trait (char)
-  `(if (typep ,char 'base-char)
-       (elt *constituent-trait-table* (char-code ,char))
-       +char-attr-constituent+))
+(declaim (inline get-constituent-trait))
+(defun get-constituent-trait (char)
+  (if (typep char 'base-char)
+      (elt *constituent-trait-table* (char-code char))
+      +char-attr-constituent+))
 
-;;;; readtable operations
+;;;; Readtable Operations
+
+(defun assert-not-standard-readtable (readtable operation)
+  (when (eq readtable *standard-readtable*)
+    (cerror "Frob it anyway!" 'standard-readtable-modified-error
+            :operation operation)))
+
+(defun readtable-case (readtable)
+  (%readtable-case readtable))
+
+(defun (setf readtable-case) (case readtable)
+  (assert-not-standard-readtable readtable '(setf readtable-case))
+  (setf (%readtable-case readtable) case))
 
 (defun shallow-replace/eql-hash-table (to from)
   (maphash (lambda (k v) (setf (gethash k to) v)) from))
 
-(defun copy-readtable (&optional (from-readtable *readtable*)
-                                 to-readtable)
+(defun copy-readtable (&optional (from-readtable *readtable*) to-readtable)
+  (assert-not-standard-readtable to-readtable 'copy-readtable)
   (let ((really-from-readtable (or from-readtable *standard-readtable*))
         (really-to-readtable (or to-readtable (make-readtable))))
     (replace (character-attribute-array really-to-readtable)
@@ -217,11 +235,12 @@
     really-to-readtable))
 
 (defun set-syntax-from-char (to-char from-char &optional
-                             (to-readtable *readtable*) (from-readtable ()))
+                             (to-readtable *readtable*) (from-readtable nil))
   #!+sb-doc
   "Causes the syntax of TO-CHAR to be the same as FROM-CHAR in the optional
 readtable (defaults to the current readtable). The FROM-TABLE defaults to the
 standard Lisp readtable when NIL."
+  (assert-not-standard-readtable to-readtable 'set-syntax-from-char)
   (let ((really-from-readtable (or from-readtable *standard-readtable*)))
     (let ((att (get-cat-entry from-char really-from-readtable))
           (mac (get-raw-cmt-entry from-char really-from-readtable))
@@ -246,12 +265,13 @@ standard Lisp readtable when NIL."
 
 (defun set-macro-character (char function &optional
                                  (non-terminatingp nil)
-                                 (readtable *readtable*))
+                                 (rt-designator *readtable*))
   #!+sb-doc
   "Causes CHAR to be a macro character which invokes FUNCTION when seen
    by the reader. The NON-TERMINATINGP flag can be used to make the macro
    character non-terminating, i.e. embeddable in a symbol name."
-  (let ((designated-readtable (or readtable *standard-readtable*)))
+  (let ((designated-readtable (or rt-designator *standard-readtable*)))
+    (assert-not-standard-readtable designated-readtable 'set-macro-character)
     (set-cat-entry char (if non-terminatingp
                             +char-attr-constituent+
                             +char-attr-terminating-macro+)
@@ -259,13 +279,13 @@ standard Lisp readtable when NIL."
     (set-cmt-entry char function designated-readtable)
     t)) ; (ANSI-specified return value)
 
-(defun get-macro-character (char &optional (readtable *readtable*))
+(defun get-macro-character (char &optional (rt-designator *readtable*))
   #!+sb-doc
   "Return the function associated with the specified CHAR which is a macro
   character, or NIL if there is no such function. As a second value, return
   T if CHAR is a macro character which is non-terminating, i.e. which can
   be embedded in a symbol name."
-  (let* ((designated-readtable (or readtable *standard-readtable*))
+  (let* ((designated-readtable (or rt-designator *standard-readtable*))
          ;; the first return value: a FUNCTION if CHAR is a macro
          ;; character, or NIL otherwise
          (fun-value (get-raw-cmt-entry char designated-readtable)))
@@ -279,11 +299,67 @@ standard Lisp readtable when NIL."
                 ;; I.e. this value means not just "non-terminating
                 ;; character?" but "non-terminating macro character?".
                 nil))))
+
+
+(defun make-char-dispatch-table ()
+  (make-hash-table))
+
+(defun make-dispatch-macro-character (char &optional
+                                      (non-terminating-p nil)
+                                      (rt *readtable*))
+  #!+sb-doc
+  "Cause CHAR to become a dispatching macro character in readtable (which
+   defaults to the current readtable). If NON-TERMINATING-P, the char will
+   be non-terminating."
+  ;; Checks already for standard readtable modification.
+  (set-macro-character char #'read-dispatch-char non-terminating-p rt)
+  (let* ((dalist (dispatch-tables rt))
+         (dtable (cdr (find char dalist :test #'char= :key #'car))))
+    (cond (dtable
+           (error "The dispatch character ~S already exists." char))
+          (t
+           (setf (dispatch-tables rt)
+                 (push (cons char (make-char-dispatch-table)) dalist)))))
+  t)
+
+(defun set-dispatch-macro-character (disp-char sub-char function
+                                     &optional (rt-designator *readtable*))
+  #!+sb-doc
+  "Cause FUNCTION to be called whenever the reader reads DISP-CHAR
+   followed by SUB-CHAR."
+  ;; Get the dispatch char for macro (error if not there), diddle
+  ;; entry for sub-char.
+  (let* ((sub-char (char-upcase sub-char))
+         (readtable (or rt-designator *standard-readtable*)))
+    (assert-not-standard-readtable readtable 'set-dispatch-macro-character)
+    (when (digit-char-p sub-char)
+      (error "SUB-CHAR must not be a decimal digit: ~S" sub-char))
+    (let ((dpair (find disp-char (dispatch-tables readtable)
+                       :test #'char= :key #'car)))
+      (if dpair
+          (setf (gethash sub-char (cdr dpair)) (coerce function 'function))
+          (error "~S is not a dispatch char." disp-char))))
+  t)
+
+(defun get-dispatch-macro-character (disp-char sub-char
+                                     &optional (rt-designator *readtable*))
+  #!+sb-doc
+  "Return the macro character function for SUB-CHAR under DISP-CHAR
+   or NIL if there is no associated function."
+  (let* ((sub-char  (char-upcase sub-char))
+         (readtable (or rt-designator *standard-readtable*))
+         (dpair     (find disp-char (dispatch-tables readtable)
+                          :test #'char= :key #'car)))
+    (if dpair
+        (values (gethash sub-char (cdr dpair)))
+        (error "~S is not a dispatch char." disp-char))))
+
 
 ;;;; definitions to support internal programming conventions
 
-(defmacro eofp (char)
-  `(eq ,char *eof-object*))
+(declaim (inline eofp))
+(defun eofp (char)
+  (eq char *eof-object*))
 
 (defun flush-whitespace (stream)
   ;; This flushes whitespace chars, returning the last char it read (a
@@ -321,98 +397,78 @@ standard Lisp readtable when NIL."
 
 ;;;; temporary initialization hack
 
+;; Install the (easy) standard macro-chars into *READTABLE*.
 (defun !cold-init-standard-readtable ()
-  (setq *standard-readtable* (make-readtable))
+  (/show0 "entering !cold-init-standard-readtable")
   ;; All characters get boring defaults in MAKE-READTABLE. Now we
   ;; override the boring defaults on characters which need more
   ;; interesting behavior.
-  (let ((*readtable* *standard-readtable*))
+  (flet ((whitespaceify (char)
+           (set-cmt-entry char nil)
+           (set-cat-entry char +char-attr-whitespace+)))
+    (whitespaceify (code-char tab-char-code))
+    (whitespaceify #\Newline)
+    (whitespaceify #\Space)
+    (whitespaceify (code-char form-feed-char-code))
+    (whitespaceify (code-char return-char-code)))
 
-    (flet ((whitespaceify (char)
-             (set-cmt-entry char nil)
-             (set-cat-entry char +char-attr-whitespace+)))
-      (whitespaceify (code-char tab-char-code))
-      (whitespaceify #\Newline)
-      (whitespaceify #\Space)
-      (whitespaceify (code-char form-feed-char-code))
-      (whitespaceify (code-char return-char-code)))
+  (set-cat-entry #\\ +char-attr-single-escape+)
+  (set-cmt-entry #\\ nil)
 
-    (set-cat-entry #\\ +char-attr-single-escape+)
-    (set-cmt-entry #\\ nil)
+  (set-cat-entry #\| +char-attr-multiple-escape+)
+  (set-cmt-entry #\| nil)
 
-    (set-cat-entry #\| +char-attr-multiple-escape+)
-    (set-cmt-entry #\| nil)
+  ;; Easy macro-character definitions are in this source file.
+  (set-macro-character #\" #'read-string)
+  (set-macro-character #\' #'read-quote)
+  (set-macro-character #\( #'read-list)
+  (set-macro-character #\) #'read-right-paren)
+  (set-macro-character #\; #'read-comment)
+  ;; (The hairier macro-character definitions, for #\# and #\`, are
+  ;; defined elsewhere, in their own source files.)
 
-    ;; Easy macro-character definitions are in this source file.
-    (set-macro-character #\" #'read-string)
-    (set-macro-character #\' #'read-quote)
-    (set-macro-character #\( #'read-list)
-    (set-macro-character #\) #'read-right-paren)
-    (set-macro-character #\; #'read-comment)
-    ;; (The hairier macro-character definitions, for #\# and #\`, are
-    ;; defined elsewhere, in their own source files.)
+  ;; all constituents
+  (do ((ichar 0 (1+ ichar))
+       (char))
+      ((= ichar base-char-code-limit))
+    (setq char (code-char ichar))
+    (when (constituentp char)
+      (set-cmt-entry char nil)))
 
-    ;; all constituents
-    (do ((ichar 0 (1+ ichar))
-         (char))
-        ((= ichar base-char-code-limit))
-      (setq char (code-char ichar))
-      (when (constituentp char *standard-readtable*)
-        (set-cmt-entry char nil)))))
+  (/show0 "leaving !cold-init-standard-readtable"))
 
 ;;;; implementation of the read buffer
 
 (defvar *read-buffer*)
-(defvar *read-buffer-length*)
-;;; FIXME: Is it really helpful to have *READ-BUFFER-LENGTH* be a
-;;; separate variable instead of just calculating it on the fly as
-;;; (LENGTH *READ-BUFFER*)?
 
-(defvar *inch-ptr*)
-(defvar *ouch-ptr*)
+(defvar *inch-ptr*) ; *OUCH-PTR* always points to next char to write.
+(defvar *ouch-ptr*) ; *INCH-PTR* always points to next char to read.
 
-(declaim (type index *read-buffer-length* *inch-ptr* *ouch-ptr*))
+(declaim (type index *inch-ptr* *ouch-ptr*))
 (declaim (type (simple-array character (*)) *read-buffer*))
 
-(defmacro reset-read-buffer ()
+(declaim (inline reset-read-buffer))
+(defun reset-read-buffer ()
   ;; Turn *READ-BUFFER* into an empty read buffer.
-  `(progn
-     ;; *OUCH-PTR* always points to next char to write.
-     (setq *ouch-ptr* 0)
-     ;; *INCH-PTR* always points to next char to read.
-     (setq *inch-ptr* 0)))
+  (setq *ouch-ptr* 0)
+  (setq *inch-ptr* 0))
 
-;;; FIXME I removed "THE FIXNUM"'s from OUCH-READ-BUFFER and
-;;; OUCH-UNREAD-BUFFER, check to make sure that Python really is smart
-;;; enough to make good code without them. And while I'm at it,
-;;; converting them from macros to inline functions might be good,
-;;; too.
-
-(defmacro ouch-read-buffer (char)
-  `(progn
-     ;; When buffer overflow
-     (when (>= *ouch-ptr* *read-buffer-length*)
-       ;; Size should be doubled.
-       (grow-read-buffer))
-     (setf (elt (the simple-string *read-buffer*) *ouch-ptr*) ,char)
-     (setq *ouch-ptr* (1+ *ouch-ptr*))))
-
-;;; macro to move *ouch-ptr* back one.
-(defmacro ouch-unread-buffer ()
-  '(when (> *ouch-ptr* *inch-ptr*)
-     (setq *ouch-ptr* (1- (the fixnum *ouch-ptr*)))))
+(declaim (inline ouch-read-buffer))
+(defun ouch-read-buffer (char)
+  ;; When buffer overflow
+  (let ((op *ouch-ptr*))
+    (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+    (when (>= op (length *read-buffer*))
+    ;; Size should be doubled.
+      (grow-read-buffer))
+    (setf (elt *read-buffer* op) char)
+    (setq *ouch-ptr* (1+ op))))
 
 (defun grow-read-buffer ()
   (let* ((rbl (length *read-buffer*))
          (new-length (* 2 rbl))
          (new-buffer (make-string new-length)))
-    (setq *read-buffer* (replace new-buffer *read-buffer*))
-    (setq *read-buffer-length* new-length)))
-
-(defun inchpeek-read-buffer ()
-  (if (>= (the fixnum *inch-ptr*) (the fixnum *ouch-ptr*))
-      *eof-object*
-      (elt *read-buffer* *inch-ptr*)))
+    (setq *read-buffer* (replace new-buffer *read-buffer*))))
 
 (defun inch-read-buffer ()
   (if (>= *inch-ptr* *ouch-ptr*)
@@ -421,9 +477,11 @@ standard Lisp readtable when NIL."
           (elt *read-buffer* *inch-ptr*)
         (incf *inch-ptr*))))
 
-(defmacro unread-buffer ()
-  `(decf *inch-ptr*))
+(declaim (inline unread-buffer))
+(defun unread-buffer ()
+  (decf *inch-ptr*))
 
+(declaim (inline read-unwind-read-buffer))
 (defun read-unwind-read-buffer ()
   ;; Keep contents, but make next (INCH..) return first character.
   (setq *inch-ptr* 0))
@@ -431,17 +489,25 @@ standard Lisp readtable when NIL."
 (defun read-buffer-to-string ()
   (subseq *read-buffer* 0 *ouch-ptr*))
 
-(defmacro with-reader ((&optional recursive-p) &body body)
-  #!+sb-doc
-  "If RECURSIVE-P is NIL, bind *READER-BUFFER* and its subservient
-variables to allow for nested and thread safe reading."
-  `(if ,recursive-p
-       (progn ,@body)
-       (let* ((*read-buffer* (make-string 128))
-              (*read-buffer-length* 128)
-              (*ouch-ptr* 0)
-              (*inch-ptr* 0))
-         ,@body)))
+(defmacro with-read-buffer (() &body body)
+  `(let* ((*read-buffer* (make-string 128))
+          (*ouch-ptr* 0)
+          (*inch-ptr* 0))
+     ,@body))
+
+(declaim (inline read-buffer-boundp))
+(defun read-buffer-boundp ()
+  (and (boundp '*read-buffer*)
+       (boundp '*ouch-ptr*)
+       (boundp '*inch-ptr*)))
+
+(defun check-for-recursive-read (stream recursive-p operator-name)
+  (when (and recursive-p (not (read-buffer-boundp)))
+    (simple-reader-error
+     stream
+     "~A was invoked with RECURSIVE-P being true outside ~
+      of a recursive read operation."
+     `(,operator-name))))
 
 ;;;; READ-PRESERVING-WHITESPACE, READ-DELIMITED-LIST, and READ
 
@@ -457,17 +523,10 @@ variables to allow for nested and thread safe reading."
 
 (declaim (special *standard-input*))
 
-;;; READ-PRESERVING-WHITESPACE behaves just like READ, only it makes
-;;; sure to leave terminating whitespace in the stream. (This is a
-;;; COMMON-LISP exported symbol.)
-(defun read-preserving-whitespace (&optional (stream *standard-input*)
-                                             (eof-error-p t)
-                                             (eof-value nil)
-                                             (recursivep nil))
-  #!+sb-doc
-  "Read from STREAM and return the value read, preserving any whitespace
-   that followed the object."
-  (if recursivep
+;;; Like READ-PRESERVING-WHITESPACE, but doesn't check the read buffer
+;;; for being set up properly.
+(defun %read-preserving-whitespace (stream eof-error-p eof-value recursive-p)
+  (if recursive-p
       ;; a loop for repeating when a macro returns nothing
       (loop
        (let ((char (read-char stream eof-error-p *eof-object*)))
@@ -480,9 +539,22 @@ variables to allow for nested and thread safe reading."
                   ;; Repeat if macro returned nothing.
                   (when result
                     (return (unless *read-suppress* (car result)))))))))
-      (with-reader ()
-        (let ((*sharp-equal-alist* nil))
-          (read-preserving-whitespace stream eof-error-p eof-value t)))))
+      (let ((*sharp-equal-alist* nil))
+        (with-read-buffer ()
+          (%read-preserving-whitespace stream eof-error-p eof-value t)))))
+
+;;; READ-PRESERVING-WHITESPACE behaves just like READ, only it makes
+;;; sure to leave terminating whitespace in the stream. (This is a
+;;; COMMON-LISP exported symbol.)
+(defun read-preserving-whitespace (&optional (stream *standard-input*)
+                                             (eof-error-p t)
+                                             (eof-value nil)
+                                             (recursive-p nil))
+  #!+sb-doc
+  "Read from STREAM and return the value read, preserving any whitespace
+   that followed the object."
+  (check-for-recursive-read stream recursive-p 'read-preserving-whitespace)
+  (%read-preserving-whitespace stream eof-error-p eof-value recursive-p))
 
 ;;; Return NIL or a list with one thing, depending.
 ;;;
@@ -497,18 +569,17 @@ variables to allow for nested and thread safe reading."
 
 (defun read (&optional (stream *standard-input*)
                        (eof-error-p t)
-                       (eof-value ())
-                       (recursivep ()))
+                       (eof-value nil)
+                       (recursive-p nil))
   #!+sb-doc
   "Read the next Lisp value from STREAM, and return it."
-  (let ((result (read-preserving-whitespace stream
-                                            eof-error-p
-                                            eof-value
-                                            recursivep)))
+  (check-for-recursive-read stream recursive-p 'read)
+  (let ((result (%read-preserving-whitespace stream eof-error-p eof-value
+                                             recursive-p)))
     ;; This function generally discards trailing whitespace. If you
     ;; don't want to discard trailing whitespace, call
     ;; CL:READ-PRESERVING-WHITESPACE instead.
-    (unless (or (eql result eof-value) recursivep)
+    (unless (or (eql result eof-value) recursive-p)
       (let ((next-char (read-char stream nil nil)))
         (unless (or (null next-char)
                     (whitespace[2]p next-char))
@@ -522,12 +593,20 @@ variables to allow for nested and thread safe reading."
   #!+sb-doc
   "Read Lisp values from INPUT-STREAM until the next character after a
    value's representation is ENDCHAR, and return the objects as a list."
-  (with-reader (recursive-p)
-    (do ((char (flush-whitespace input-stream)
-               (flush-whitespace input-stream))
-         (retlist ()))
-        ((char= char endchar) (unless *read-suppress* (nreverse retlist)))
-      (setq retlist (nconc (read-maybe-nothing input-stream char) retlist)))))
+  (check-for-recursive-read input-stream recursive-p 'read-delimited-list)
+  (flet ((%read-delimited-list (endchar input-stream)
+           (do ((char (flush-whitespace input-stream)
+                      (flush-whitespace input-stream))
+                (retlist ()))
+               ((char= char endchar)
+                (unless *read-suppress* (nreverse retlist)))
+             (setq retlist (nconc (read-maybe-nothing input-stream char)
+                                  retlist)))))
+    (declare (inline %read-delimited-list))
+    (if recursive-p
+        (%read-delimited-list endchar input-stream)
+        (with-read-buffer ()
+          (%read-delimited-list endchar input-stream)))))
 
 ;;;; basic readmacro definitions
 ;;;;
@@ -544,7 +623,9 @@ variables to allow for nested and thread safe reading."
       ((character-decoding-error
         #'(lambda (decoding-error)
             (declare (ignorable decoding-error))
-            (style-warn "Character decoding error in a ;-comment at position ~A reading source file ~A, resyncing." (file-position stream) stream)
+            (style-warn
+             'sb!kernel::character-decoding-error-in-macro-char-comment
+             :position (file-position stream) :stream stream)
             (invoke-restart 'attempt-resync))))
     (let ((stream (in-synonym-of stream)))
       (if (ansi-stream-p stream)
@@ -1423,10 +1504,7 @@ variables to allow for nested and thread safe reading."
                           :format-control "failed to build ratio")))))
       (if negative-number (- num) num))))
 
-;;;; cruft for dispatch macros
-
-(defun make-char-dispatch-table ()
-  (make-hash-table))
+;;;; General reader for dispatch macros
 
 (defun dispatch-char-error (stream sub-char ignore)
   (declare (ignore ignore))
@@ -1436,53 +1514,6 @@ variables to allow for nested and thread safe reading."
                            "no dispatch function defined for ~S"
                            sub-char)))
 
-(defun make-dispatch-macro-character (char &optional
-                                           (non-terminating-p nil)
-                                           (rt *readtable*))
-  #!+sb-doc
-  "Cause CHAR to become a dispatching macro character in readtable (which
-   defaults to the current readtable). If NON-TERMINATING-P, the char will
-   be non-terminating."
-  (set-macro-character char #'read-dispatch-char non-terminating-p rt)
-  (let* ((dalist (dispatch-tables rt))
-         (dtable (cdr (find char dalist :test #'char= :key #'car))))
-    (cond (dtable
-           (error "The dispatch character ~S already exists." char))
-          (t
-           (setf (dispatch-tables rt)
-                 (push (cons char (make-char-dispatch-table)) dalist)))))
-  t)
-
-(defun set-dispatch-macro-character (disp-char sub-char function
-                                               &optional (rt *readtable*))
-  #!+sb-doc
-  "Cause FUNCTION to be called whenever the reader reads DISP-CHAR
-   followed by SUB-CHAR."
-  ;; Get the dispatch char for macro (error if not there), diddle
-  ;; entry for sub-char.
-  (when (digit-char-p sub-char)
-    (error "SUB-CHAR must not be a decimal digit: ~S" sub-char))
-  (let* ((sub-char (char-upcase sub-char))
-         (rt (or rt *standard-readtable*))
-         (dpair (find disp-char (dispatch-tables rt)
-                      :test #'char= :key #'car)))
-    (if dpair
-        (setf (gethash sub-char (cdr dpair)) (coerce function 'function))
-        (error "~S is not a dispatch char." disp-char))))
-
-(defun get-dispatch-macro-character (disp-char sub-char
-                                     &optional (rt *readtable*))
-  #!+sb-doc
-  "Return the macro character function for SUB-CHAR under DISP-CHAR
-   or NIL if there is no associated function."
-  (let* ((sub-char (char-upcase sub-char))
-         (rt (or rt *standard-readtable*))
-         (dpair (find disp-char (dispatch-tables rt)
-                      :test #'char= :key #'car)))
-    (if dpair
-        (values (gethash sub-char (cdr dpair)))
-        (error "~S is not a dispatch char." disp-char))))
-
 (defun read-dispatch-char (stream char)
   ;; Read some digits.
   (let ((numargp nil)
@@ -1525,7 +1556,7 @@ variables to allow for nested and thread safe reading."
                     :check-fill-pointer t)
     (let ((stream (make-string-input-stream string start end)))
       (values (if preserve-whitespace
-                  (read-preserving-whitespace stream eof-error-p eof-value)
+                  (%read-preserving-whitespace stream eof-error-p eof-value nil)
                   (read stream eof-error-p eof-value))
               (- (string-input-stream-current stream) offset)))))
 
