@@ -17,6 +17,13 @@
 ;;;; function names and documentation
 
 ;;;; the ANSI interface to function names (and to other stuff too)
+;;; Note: this function gets called by the compiler (as of 1.0.17.x,
+;;; in MAYBE-INLINE-SYNTACTIC-CLOSURE), and so although ANSI says
+;;; we're allowed to return NIL here freely, it seems plausible that
+;;; small changes to the circumstances under which this function
+;;; returns non-NIL might have subtle consequences on the compiler.
+;;; So it might be desirable to have the compiler not rely on this
+;;; function, eventually.
 (defun function-lambda-expression (fun)
   "Return (VALUES DEFINING-LAMBDA-EXPRESSION CLOSURE-P NAME), where
   DEFINING-LAMBDA-EXPRESSION is NIL if unknown, or a suitable argument
@@ -39,9 +46,9 @@
             (info (sb!kernel:%code-debug-info code)))
        (if info
            (let ((source (sb!c::debug-info-source info)))
-             (cond ((and (eq (sb!c::debug-source-from source) :lisp)
+             (cond ((and (sb!c::debug-source-form source)
                          (eq (sb!c::debug-source-function source) fun))
-                    (values (svref (sb!c::debug-source-name source) 0)
+                    (values (sb!c::debug-source-form source)
                             nil
                             name))
                    ((legal-fun-name-p name)
