@@ -122,7 +122,7 @@
              (unless (or (constant-p v)
                          (and (global-var-p v)
                               (member (global-var-kind v)
-                                      '(:global :special))))
+                                      '(:global :special :unknown))))
                (barf "strange *FREE-VARS* entry: ~S" v))
              (dolist (n (leaf-refs v))
                (check-node-reached n))
@@ -642,7 +642,7 @@
                                    atypes)
                      (template-more-args-type info) "args")
       (check-tn-refs (vop-results vop) vop t
-                     (if (eq rtypes :conditional) 0 (length rtypes))
+                     (if (template-conditional-p info) 0 (length rtypes))
                      (template-more-results-type info) "results")
       (check-tn-refs (vop-temps vop) vop t 0 t "temps")
       (unless (= (length (vop-codegen-info vop))
@@ -1009,7 +1009,12 @@
              (case (cleanup-kind cleanup)
                ((:dynamic-extent)
                 (format t "entry DX~{ v~D~}"
-                        (mapcar #'cont-num (cleanup-info cleanup))))
+                        (mapcar (lambda (lvar-or-cell)
+                                  (if (consp lvar-or-cell)
+                                      (cons (car lvar-or-cell)
+                                            (cont-num (cdr lvar-or-cell)))
+                                      (cont-num lvar-or-cell)))
+                                (cleanup-info cleanup))))
                (t
                 (format t "entry ~S" (entry-exits node))))))
           (exit
