@@ -88,7 +88,7 @@
 (defvar *compiler-style-warning-count*)
 (defvar *compiler-note-count*)
 (defvar *compiler-trace-output*)
-(defvar *constraint-number*)
+(defvar *constraint-universe*)
 (defvar *count-vop-usages*)
 (defvar *current-path*)
 (defvar *current-component*)
@@ -106,15 +106,30 @@
 (defvar *info-environment*)
 (defvar *lexenv*)
 (defvar *source-info*)
+(defvar *source-plist*)
 (defvar *trace-table*)
 (defvar *undefined-warnings*)
 (defvar *warnings-p*)
 (defvar *lambda-conversions*)
 
-;;; This lock is seized in the compiler, and related areas: the
-;;; compiler is not presently thread-safe
-(defvar *big-compiler-lock*
-  (sb!thread:make-mutex :name "big compiler lock"))
+(defvar *stack-allocate-dynamic-extent* t
+  "If true (the default), the compiler respects DYNAMIC-EXTENT declarations
+and stack allocates otherwise inaccessible parts of the object whenever
+possible. Potentially long (over one page in size) vectors are, however, not
+stack allocated except in zero SAFETY code, as such a vector could overflow
+the stack without triggering overflow protection.")
+
+(!begin-collecting-cold-init-forms)
+;;; This lock is seized in the compiler, and related areas -- like the
+;;; classoid/layout/class system.
+(defvar *world-lock*)
+(!cold-init-forms
+ (setf *world-lock* (sb!thread:make-mutex :name "World Lock")))
+(!defun-from-collected-cold-init-forms !world-lock-cold-init)
+
+(defmacro with-world-lock (() &body body)
+  `(sb!thread:with-recursive-lock (*world-lock*)
+     ,@body))
 
 (declaim (type fixnum *compiler-sset-counter*))
 (defvar *compiler-sset-counter* 0)
@@ -164,9 +179,11 @@
     ;; and then we happen to compile bar.lisp before foo.lisp.
   (when (looks-like-name-of-special-var-p symbol)
     ;; FIXME: should be COMPILER-STYLE-WARNING?
-    (style-warn "using the lexical binding of the symbol ~S, not the~@
-dynamic binding, even though the symbol name follows the usual naming~@
-convention (names like *FOO*) for special variables" symbol))
+    (style-warn 'sb!kernel:asterisks-around-lexical-variable-name
+                :format-control
+                "using the lexical binding of the symbol ~S, not the~@
+                 dynamic binding"
+                :format-arguments (list symbol)))
   (values))
 
 (def!struct (debug-name-marker (:make-load-form-fun dump-debug-name-marker)
@@ -178,7 +195,7 @@ convention (names like *FOO*) for special variables" symbol))
 (defvar *debug-name-sharp*)
 (defvar *debug-name-ellipsis*)
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
+(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
   (defun dump-debug-name-marker (marker &optional env)
     (declare (ignore env))
     (cond ((eq marker *debug-name-sharp*)
