@@ -43,11 +43,11 @@
              (setf (car args) nil)))
   (values))
 
-(defun handle-nested-dynamic-extent-lvars (lvar)
+(defun handle-nested-dynamic-extent-lvars (dx lvar)
   (let ((uses (lvar-uses lvar)))
-    ;; Stack analysis wants DX value generators to end their
-    ;; blocks. Uses of mupltiple used LVARs already end their blocks,
-    ;; so we just need to process used-once LVARs.
+    ;; DX value generators must end their blocks: see UPDATE-UVL-LIVE-SETS.
+    ;; Uses of mupltiple-use LVARs already end their blocks, so we just need
+    ;; to process uses of single-use LVARs.
     (when (node-p uses)
       (node-ends-block uses))
     ;; If this LVAR's USE is good for DX, it is either a CAST, or it
@@ -55,51 +55,81 @@
     (flet ((recurse (use)
              (etypecase use
                (cast
-                (handle-nested-dynamic-extent-lvars (cast-value use)))
+                (handle-nested-dynamic-extent-lvars dx (cast-value use)))
                (combination
                 (loop for arg in (combination-args use)
-                      when (lvar-good-for-dx-p arg)
-                      append (handle-nested-dynamic-extent-lvars arg))))))
-      (cons lvar
+                      ;; deleted args show up as NIL here
+                      when (and arg (lvar-good-for-dx-p arg dx))
+                      append (handle-nested-dynamic-extent-lvars dx arg)))
+               (ref
+                (let* ((other (trivial-lambda-var-ref-lvar use)))
+                  (unless (eq other lvar)
+                    (handle-nested-dynamic-extent-lvars dx other)))))))
+      (cons (cons dx lvar)
             (if (listp uses)
                 (loop for use in uses
-                      when (use-good-for-dx-p use)
+                      when (use-good-for-dx-p use dx)
                       nconc (recurse use))
-                (when (use-good-for-dx-p uses)
+                (when (use-good-for-dx-p uses dx)
                   (recurse uses)))))))
 
 (defun recognize-dynamic-extent-lvars (call fun)
   (declare (type combination call) (type clambda fun))
   (loop for arg in (basic-combination-args call)
-        and var in (lambda-vars fun)
-        when (and arg (lambda-var-dynamic-extent var)
-                  (not (lvar-dynamic-extent arg)))
-        append (handle-nested-dynamic-extent-lvars arg) into dx-lvars
+        for var in (lambda-vars fun)
+        for dx = (lambda-var-dynamic-extent var)
+        when (and dx arg (not (lvar-dynamic-extent arg)))
+        append (handle-nested-dynamic-extent-lvars dx arg) into dx-lvars
         finally (when dx-lvars
-                  (binding* ((before-ctran (node-prev call))
-                             (nil (ensure-block-start before-ctran))
-                             (block (ctran-block before-ctran))
-                             (new-call-ctran (make-ctran :kind :inside-block
-                                                         :next call
-                                                         :block block))
-                             (entry (with-ir1-environment-from-node call
-                                      (make-entry :prev before-ctran
-                                                  :next new-call-ctran)))
-                             (cleanup (make-cleanup :kind :dynamic-extent
-                                                    :mess-up entry
-                                                    :info dx-lvars)))
-                    (setf (node-prev call) new-call-ctran)
-                    (setf (ctran-next before-ctran) entry)
-                    (setf (ctran-use new-call-ctran) entry)
+                  ;; Stack analysis requires that the CALL ends the block, so
+                  ;; that MAP-BLOCK-NLXES sees the cleanup we insert here.
+                  (node-ends-block call)
+                  (let* ((entry (with-ir1-environment-from-node call
+                                  (make-entry)))
+                         (cleanup (make-cleanup :kind :dynamic-extent
+                                                :mess-up entry
+                                                :info dx-lvars)))
                     (setf (entry-cleanup entry) cleanup)
+                    (insert-node-before call entry)
                     (setf (node-lexenv call)
                           (make-lexenv :default (node-lexenv call)
                                        :cleanup cleanup))
                     (push entry (lambda-entries (node-home-lambda entry)))
-                    (dolist (lvar dx-lvars)
-                      (setf (lvar-dynamic-extent lvar) cleanup)))))
+                    (dolist (cell dx-lvars)
+                      (setf (lvar-dynamic-extent (cdr cell)) cleanup)))))
   (values))
 
+;;; Called after a transform has been applied to CALL: if the call has a DX
+;;; result, propagate the DXness to the new functional as well.
+;;;
+;;; This is needed in case an earlier call to LOCALL-ANALYZE-COMPONENT
+;;; collected DX information before the transformation, in which case a later
+;;; call to LOCALL-ANALYZE-COMPONENT would not pick up the DX declaration
+;;; again, since the call has already been converted. (In other words, work
+;;; around the fact that optimization iterates, and locall analysis may have
+;;; already run by the time we are able to transform something.)
+(defun maybe-propagate-dynamic-extent (call fun)
+  (when (lambda-p fun)
+    (let* ((lvar (combination-lvar call))
+           (cleanup (or (and lvar (lvar-dynamic-extent lvar))
+                        (return-from maybe-propagate-dynamic-extent)))
+           (ret (lambda-return fun))
+           (res (if ret
+                    (return-result ret)
+                    (return-from maybe-propagate-dynamic-extent)))
+           (dx (car (rassoc lvar (cleanup-info cleanup) :test #'eq)))
+           (new-dx-lvars (if (and dx res)
+                             (handle-nested-dynamic-extent-lvars dx res)
+                             (return-from maybe-propagate-dynamic-extent))))
+      (when new-dx-lvars
+        ;; This builds on what RECOGNIZE-DYNAMIC-EXTENT-LVARS does above.
+        (aver (eq call (block-last (node-block call))))
+        (dolist (cell new-dx-lvars)
+          (let ((lvar (cdr cell)))
+            (aver (not (lvar-dynamic-extent lvar)))
+            (push cell (cleanup-info cleanup))
+            (setf (lvar-dynamic-extent (cdr cell)) cleanup)))))))
+
 ;;; This function handles merging the tail sets if CALL is potentially
 ;;; tail-recursive, and is a call to a function with a different
 ;;; TAIL-SET than CALL's FUN. This must be called whenever we alter
@@ -211,9 +241,6 @@
                do (entries `((eql ,n-supplied ,n)
                              (%funcall ,(force ep) ,@(subseq temps 0 n)))))
          `(lambda (,n-supplied ,@temps)
-            ;; FIXME: Make sure that INDEX type distinguishes between
-            ;; target and host. (Probably just make the SB!XC:DEFTYPE
-            ;; different from CL:DEFTYPE.)
             (declare (type index ,n-supplied))
             (cond
              ,@(if more (butlast (entries)) (entries))
@@ -221,8 +248,7 @@
                  ;; KLUDGE: (NOT (< ...)) instead of >= avoids one round of
                  ;; deftransforms and lambda-conversion.
                  `((,(if (zerop min) t `(not (< ,n-supplied ,max)))
-                    ,(let ((n-context (gensym))
-                           (n-count (gensym)))
+                    ,(with-unique-names (n-context n-count)
                        `(multiple-value-bind (,n-context ,n-count)
                             (%more-arg-context ,n-supplied ,max)
                           (locally
@@ -246,7 +272,8 @@
   (with-ir1-environment-from-node (lambda-bind (main-entry fun))
     (let ((res (ir1-convert-lambda (make-xep-lambda-expression fun)
                                    :debug-name (debug-name
-                                                'xep (leaf-debug-name fun)))))
+                                                'xep (leaf-debug-name fun))
+                                   :system-lambda t)))
       (setf (functional-kind res) :external
             (leaf-ever-used res) t
             (functional-entry-fun res) fun
@@ -601,7 +628,8 @@
                (%funcall ,entry ,@args))
             :debug-name (debug-name 'hairy-function-entry
                                     (lvar-fun-debug-name
-                                     (basic-combination-fun call)))))))
+                                     (basic-combination-fun call)))
+            :system-lambda t))))
     (convert-call ref call new-fun)
     (dolist (ref (leaf-refs entry))
       (convert-call-if-possible ref (lvar-dest (node-lvar ref))))))
@@ -861,6 +889,8 @@
     (setf (lambda-physenv clambda) home-physenv)
 
     (when physenv
+      (unless home-physenv
+        (setf home-physenv (get-lambda-physenv home)))
       (setf (physenv-nlx-info home-physenv)
             (nconc (physenv-nlx-info physenv)
                    (physenv-nlx-info home-physenv))))
