@@ -24,11 +24,6 @@
 ;;; object
 (def!constant n-byte-bits 8)
 
-
-;;; The size in bytes of the GENCGC pages. Should be a multiple of the
-;;; architecture code size.
-(def!constant gencgc-page-size 4096)
-
 ;;; flags for the generational garbage collector
 (def!constant pseudo-atomic-interrupted-flag 1)
 (def!constant pseudo-atomic-flag 4)
@@ -90,13 +85,31 @@
 (def!constant float-fast-bit 2)         ; Non-IEEE mode
 
 
-;;; Where to put the different spaces.
+;;;; Where to put the different spaces.
 
-(def!constant read-only-space-start #x04000000)
-(def!constant read-only-space-end   #x07ff8000)
+;;; On non-gencgc we need large dynamic and static spaces for PURIFY
+#!-gencgc
+(progn
+  (def!constant read-only-space-start #x04000000)
+  (def!constant read-only-space-end   #x07ff8000)
+  (def!constant static-space-start    #x08000000)
+  (def!constant static-space-end      #x097fff00)
 
-(def!constant static-space-start    #x08000000)
-(def!constant static-space-end      #x097fff00)
+  (def!constant linkage-table-space-start #x0a000000)
+  (def!constant linkage-table-space-end   #x0b000000))
+
+;;; While on gencgc we don't.
+#!+gencgc
+(progn
+  (def!constant read-only-space-start #x04000000)
+  (def!constant read-only-space-end   #x040ff000)
+  (def!constant static-space-start    #x04100000)
+  (def!constant static-space-end      #x041ff000)
+
+  (def!constant linkage-table-space-start #x04200000)
+  (def!constant linkage-table-space-end   #x042ff000))
+
+(def!constant linkage-table-entry-size 16)
 
 #!+linux
 (progn
@@ -109,11 +122,20 @@
     (def!constant dynamic-0-space-start #x4f000000)
     (def!constant dynamic-0-space-end   #x66fff000)
     (def!constant dynamic-1-space-start #x67000000)
-    (def!constant dynamic-1-space-end   #x7efff000))
+    (def!constant dynamic-1-space-end   #x7efff000)))
 
-  (def!constant linkage-table-space-start #x0a000000)
-  (def!constant linkage-table-space-end   #x0b000000)
-  (def!constant linkage-table-entry-size 16))
+#!+netbsd
+(progn
+  #!+gencgc
+  (progn
+    (def!constant dynamic-space-start #x4f000000)
+    (def!constant dynamic-space-end   #x7efff000))
+  #!-gencgc
+  (progn
+    (def!constant dynamic-0-space-start #x4f000000)
+    (def!constant dynamic-0-space-end   #x66fff000)
+    (def!constant dynamic-1-space-start #x67000000)
+    (def!constant dynamic-1-space-end   #x7efff000)))
 
 #!+darwin
 (progn
@@ -127,37 +149,25 @@
     (def!constant dynamic-0-space-end   #x3ffff000)
 
     (def!constant dynamic-1-space-start #x40000000)
-    (def!constant dynamic-1-space-end   #x6ffff000))
-
-
-  (def!constant linkage-table-space-start #x0a000000)
-  (def!constant linkage-table-space-end   #x0b000000)
-  (def!constant linkage-table-entry-size 16))
+    (def!constant dynamic-1-space-end   #x6ffff000)))
 
 ;;;; Other miscellaneous constants.
 
-(defenum (:suffix -trap :start 8)
-  halt
-  pending-interrupt
-  error
-  cerror
-  breakpoint
-  fun-end-breakpoint
-  after-breakpoint
-  fixnum-additive-overflow
-  single-step-around
-  single-step-before)
-
-(defenum (:prefix object-not- :suffix -trap :start 24)
-  list
-  instance)
-
-(defenum (:prefix trace-table-)
-  normal
-  call-site
-  fun-prologue
-  fun-epilogue)
+(defenum (:start 8)
+  halt-trap
+  pending-interrupt-trap
+  error-trap
+  cerror-trap
+  breakpoint-trap
+  fun-end-breakpoint-trap
+  after-breakpoint-trap
+  fixnum-additive-overflow-trap
+  single-step-around-trap
+  single-step-before-trap)
 
+(defenum (:start 24)
+  object-not-list-trap
+  object-not-instance-trap)
 
 ;;;; Static symbols.
 
