@@ -1008,8 +1008,8 @@
 ;;;; binary conditional VOPs
 
 (define-vop (fast-conditional)
-  (:conditional)
-  (:info target not-p)
+  (:conditional :e)
+  (:info)
   (:effects)
   (:affected)
   (:policy :fast-safe))
@@ -1028,7 +1028,7 @@
 (define-vop (fast-conditional-c/fixnum fast-conditional/fixnum)
   (:args (x :scs (any-reg control-stack)))
   (:arg-types tagged-num (:constant (signed-byte 29)))
-  (:info target not-p y))
+  (:info y))
 
 (define-vop (fast-conditional/signed fast-conditional)
   (:args (x :scs (signed-reg)
@@ -1041,7 +1041,7 @@
 (define-vop (fast-conditional-c/signed fast-conditional/signed)
   (:args (x :scs (signed-reg signed-stack)))
   (:arg-types signed-num (:constant (signed-byte 31)))
-  (:info target not-p y))
+  (:info y))
 
 (define-vop (fast-conditional/unsigned fast-conditional)
   (:args (x :scs (unsigned-reg)
@@ -1054,7 +1054,7 @@
 (define-vop (fast-conditional-c/unsigned fast-conditional/unsigned)
   (:args (x :scs (unsigned-reg unsigned-stack)))
   (:arg-types unsigned-num (:constant (unsigned-byte 31)))
-  (:info target not-p y))
+  (:info y))
 
 (macrolet ((define-conditional-vop (tran cond unsigned not-cond not-unsigned)
              `(progn
@@ -1068,19 +1068,12 @@
                                      (format nil "~:@(FAST-CONDITIONAL~A~)"
                                              suffix)))
                         (:translate ,tran)
+                        (:conditional ,(if signed cond unsigned))
                         (:generator ,cost
                                     (inst cmp x
                                           ,(if (eq suffix '-c/fixnum)
                                                '(fixnumize y)
-                                               'y))
-                                    (inst jmp (if not-p
-                                                  ,(if signed
-                                                       not-cond
-                                                       not-unsigned)
-                                                  ,(if signed
-                                                       cond
-                                                       unsigned))
-                                          target))))
+                                               'y)))))
                    '(/fixnum -c/fixnum /signed -c/signed /unsigned -c/unsigned)
 ;                  '(/fixnum  /signed  /unsigned)
                    '(4 3 6 5 6 5)
@@ -1092,8 +1085,7 @@
 (define-vop (fast-if-eql/signed fast-conditional/signed)
   (:translate eql)
   (:generator 6
-    (inst cmp x y)
-    (inst jmp (if not-p :ne :e) target)))
+    (inst cmp x y)))
 
 (define-vop (fast-if-eql-c/signed fast-conditional-c/signed)
   (:translate eql)
@@ -1101,14 +1093,12 @@
     (cond ((and (sc-is x signed-reg) (zerop y))
            (inst test x x))  ; smaller instruction
           (t
-           (inst cmp x y)))
-    (inst jmp (if not-p :ne :e) target)))
+           (inst cmp x y)))))
 
 (define-vop (fast-if-eql/unsigned fast-conditional/unsigned)
   (:translate eql)
   (:generator 6
-    (inst cmp x y)
-    (inst jmp (if not-p :ne :e) target)))
+    (inst cmp x y)))
 
 (define-vop (fast-if-eql-c/unsigned fast-conditional-c/unsigned)
   (:translate eql)
@@ -1116,8 +1106,7 @@
     (cond ((and (sc-is x unsigned-reg) (zerop y))
            (inst test x x))  ; smaller instruction
           (t
-           (inst cmp x y)))
-    (inst jmp (if not-p :ne :e) target)))
+           (inst cmp x y)))))
 
 ;;; EQL/FIXNUM is funny because the first arg can be of any type, not just a
 ;;; known fixnum.
@@ -1137,8 +1126,8 @@
   (:note "inline fixnum comparison")
   (:translate eql)
   (:generator 4
-    (inst cmp x y)
-    (inst jmp (if not-p :ne :e) target)))
+    (inst cmp x y)))
+
 (define-vop (generic-eql/fixnum fast-eql/fixnum)
   (:args (x :scs (any-reg descriptor-reg)
             :load-if (not (and (sc-is x control-stack)
@@ -1147,18 +1136,16 @@
   (:arg-types * tagged-num)
   (:variant-cost 7))
 
-
 (define-vop (fast-eql-c/fixnum fast-conditional/fixnum)
   (:args (x :scs (any-reg control-stack)))
   (:arg-types tagged-num (:constant (signed-byte 29)))
-  (:info target not-p y)
+  (:info y)
   (:translate eql)
   (:generator 2
     (cond ((and (sc-is x any-reg) (zerop y))
            (inst test x x))  ; smaller instruction
           (t
-           (inst cmp x (fixnumize y))))
-    (inst jmp (if not-p :ne :e) target)))
+           (inst cmp x (fixnumize y))))))
 
 (define-vop (generic-eql-c/fixnum fast-eql-c/fixnum)
   (:args (x :scs (any-reg descriptor-reg control-stack)))
@@ -1167,21 +1154,6 @@
 
 ;;;; 32-bit logical operations
 
-(define-vop (merge-bits)
-  (:translate merge-bits)
-  (:args (shift :scs (signed-reg unsigned-reg) :target ecx)
-         (prev :scs (unsigned-reg) :target result)
-         (next :scs (unsigned-reg)))
-  (:arg-types tagged-num unsigned-num unsigned-num)
-  (:temporary (:sc signed-reg :offset ecx-offset :from (:argument 0)) ecx)
-  (:results (result :scs (unsigned-reg) :from (:argument 1)))
-  (:result-types unsigned-num)
-  (:policy :fast-safe)
-  (:generator 4
-    (move ecx shift)
-    (move result prev)
-    (inst shrd result next :cl)))
-
 ;;; Only the lower 6 bits of the shift amount are significant.
 (define-vop (shift-towards-someplace)
   (:policy :fast-safe)
@@ -1413,11 +1385,9 @@
   (:policy :fast-safe)
   (:args (digit :scs (unsigned-reg)))
   (:arg-types unsigned-num)
-  (:conditional)
-  (:info target not-p)
+  (:conditional :ns)
   (:generator 3
-    (inst or digit digit)
-    (inst jmp (if not-p :s :ns) target)))
+    (inst or digit digit)))
 
 
 ;;; For add and sub with carry the sc of carry argument is any-reg so
