@@ -159,21 +159,27 @@
 ;;;     And if KVA_PAGES is extended from 1GB to 1.5GB, we can't use
 ;;;     down to around 0xA0000000.
 ;;;     So we use 0x58000000--0x98000000 for dynamic space.
+;;;   * OpenBSD address space changes for W^X as well as malloc
+;;;     randomization made the old addresses unsafe. The only range
+;;;     that is really safe is between the end of the text segment (it
+;;;     starts at #x3C000000) and #x7C000000. However if the -Z linker
+;;;     option is used then the safe range is (probably) #x00001000 to
+;;;     #x48048000, with the text and data segments at #x08048000.
 
 #!+win32
 (progn
 
-  (def!constant read-only-space-start #x02000000)
-  (def!constant read-only-space-end   #x020ff000)
+  (def!constant read-only-space-start #x22000000)
+  (def!constant read-only-space-end   #x220ff000)
 
-  (def!constant static-space-start    #x02100000)
-  (def!constant static-space-end      #x021ff000)
+  (def!constant static-space-start    #x22100000)
+  (def!constant static-space-end      #x221ff000)
 
-  (def!constant dynamic-space-start   #x09000000)
-  (def!constant dynamic-space-end     #x29000000)
+  (def!constant dynamic-space-start   #x22300000)
+  (def!constant dynamic-space-end     #x42300000)
 
-  (def!constant linkage-table-space-start #x02200000)
-  (def!constant linkage-table-space-end   #x022ff000))
+  (def!constant linkage-table-space-start #x22200000)
+  (def!constant linkage-table-space-end   #x222ff000))
 
 #!+linux
 (progn
@@ -219,18 +225,18 @@
 
 #!+openbsd
 (progn
-  (def!constant read-only-space-start     #x10000000)
-  (def!constant read-only-space-end       #x100ff000)
+  (def!constant read-only-space-start     #x47000000)
+  (def!constant read-only-space-end       #x470ff000)
 
-  (def!constant static-space-start        #x10100000)
-  (def!constant static-space-end          #x101ff000)
+  (def!constant static-space-start        #x47100000)
+  (def!constant static-space-end          #x471ff000)
 
-  (def!constant dynamic-space-start       #x80000000)
-  (def!constant dynamic-space-end         #xA0000000)
+  (def!constant dynamic-space-start       #x48000000)
+  (def!constant dynamic-space-end         #x78000000)
 
   ;; In CMUCL: 0xB0000000->0xB1000000
-  (def!constant linkage-table-space-start #x10200000)
-  (def!constant linkage-table-space-end   #x102ff000))
+  (def!constant linkage-table-space-start #x47200000)
+  (def!constant linkage-table-space-end   #x472ff000))
 
 #!+netbsd
 (progn
@@ -271,32 +277,19 @@
 
 ;;;; other miscellaneous constants
 
-(defenum (:suffix -trap :start 8)
-  halt
-  pending-interrupt
-  error
-  cerror
-  breakpoint
-  fun-end-breakpoint
-  single-step-around
-  single-step-before)
-;;; FIXME: It'd be nice to replace all the DEFENUMs with something like
-;;;   (WITH-DEF-ENUM (:START 8)
-;;;     (DEF-ENUM HALT-TRAP)
-;;;     (DEF-ENUM PENDING-INTERRUPT-TRAP)
-;;;     ..)
-;;; for the benefit of anyone doing a lexical search for definitions
-;;; of these symbols.
+(defenum (:start 8)
+  halt-trap
+  pending-interrupt-trap
+  error-trap
+  cerror-trap
+  breakpoint-trap
+  fun-end-breakpoint-trap
+  single-step-around-trap
+  single-step-before-trap)
 
-(defenum (:prefix object-not- :suffix -trap :start 24)
-  list
-  instance)
-
-(defenum (:prefix trace-table-)
-  normal
-  call-site
-  fun-prologue
-  fun-epilogue)
+(defenum (:start 24)
+  object-not-list-trap
+  object-not-instance-trap)
 
 ;;;; static symbols
 
