@@ -47,6 +47,9 @@
 (defmethod documentation ((x symbol) (doc-type (eql 'setf)))
   (values (info :setf :documentation x)))
 
+(defmethod documentation ((x symbol) (doc-type (eql 'optimize)))
+  (random-documentation x 'optimize))
+
 (defun (setf fun-doc) (new-value x)
   (etypecase x
     (generic-function
@@ -57,7 +60,7 @@
            new-value))
     (function
      (let ((name (%fun-name x)))
-       (when (and name (typep name '(or symbol cons)))
+       (when (valid-function-name-p name)
          (setf (info :function :documentation name) new-value)))))
   new-value)
 
