@@ -257,6 +257,8 @@
 
 (defgeneric finalize-inheritance (class))
 
+(defgeneric function-keywords (method))
+
 (defgeneric function-keyword-parameters (method))
 
 (defgeneric generic-function-argument-precedence-order (gf))
@@ -361,8 +363,6 @@
 
 (defgeneric unparse-specializer-using-class (generic-function specializer))
 
-(defgeneric update-gf-dfun (class gf))
-
 (defgeneric validate-superclass (class superclass))
 
 (defgeneric (setf documentation) (new-value slotd doc-type)
@@ -411,12 +411,6 @@
 
 ;;;; 4 arguments
 
-(defgeneric add-boundp-method (class generic-function slot-name slot-documentation))
-
-(defgeneric add-reader-method (class generic-function slot-name slot-documentation))
-
-(defgeneric add-writer-method (class generic-function slot-name slot-documentation))
-
 (defgeneric make-method-lambda
     (proto-generic-function proto-method lambda-expression environment))
 
@@ -427,6 +421,14 @@
 
 ;;;; 5 arguments
 
+;;; FIXME: This is currently unused -- where should we call it? Or should we just
+;;; delete it.
+(defgeneric add-boundp-method (class generic-function slot-name slot-documentation source-location))
+
+(defgeneric add-reader-method (class generic-function slot-name slot-documentation source-location))
+
+(defgeneric add-writer-method (class generic-function slot-name slot-documentation source-location))
+
 (defgeneric make-method-initargs-form
     (proto-generic-function proto-method lambda-expression lambda-list
      environment))
@@ -464,7 +466,7 @@
                                                  fun-name
                                                  &key &allow-other-keys))
 
-(defgeneric initialize-instance (gf &key &allow-other-keys))
+(defgeneric initialize-instance (instance &rest initargs &key &allow-other-keys))
 
 (defgeneric make-instance (class &rest initargs &key &allow-other-keys))
 
@@ -478,10 +480,9 @@
 
 (defgeneric reader-method-class (class direct-slot &rest initargs))
 
-(defgeneric reinitialize-instance (gf &rest args &key &allow-other-keys))
+(defgeneric reinitialize-instance (instance &rest initargs &key &allow-other-keys))
 
-(defgeneric shared-initialize (generic-function
-                               slot-names
+(defgeneric shared-initialize (instance slot-names &rest initargs
                                &key &allow-other-keys))
 
 (defgeneric update-dependent (metaobject dependent &rest initargs))
