@@ -5,6 +5,8 @@
 
 #include <mach/mach_init.h>
 #include <mach/task.h>
+#include <AvailabilityMacros.h>
+#include <sys/cdefs.h>
 
 /* man pages claim that the third argument is a sigcontext struct,
    but ucontext_t is defined, matches sigcontext where sensible,
@@ -16,7 +18,13 @@
 #if defined(LISP_FEATURE_X86)
 #include <sys/ucontext.h>
 #include <sys/_types.h>
+
+#if __DARWIN_UNIX03
+typedef struct __darwin_ucontext os_context_t;
+#else
 typedef struct ucontext os_context_t;
+#endif
+
 
 #else
 #include <ucontext.h>
@@ -25,8 +33,6 @@ typedef ucontext_t os_context_t;
 
 #define SIG_MEMORY_FAULT SIGBUS
 
-#define SIG_INTERRUPT_THREAD (SIGINFO)
-#define SIG_STOP_FOR_GC (SIGUSR1)
-#define SIG_RESUME_FROM_GC (SIGUSR2)
+#define SIG_STOP_FOR_GC (SIGUSR2)
 
 #endif /* _DARWIN_OS_H */
