@@ -122,10 +122,8 @@ is_valid_lisp_addr(os_vm_address_t addr)
 
 
 static void
-sigsegv_handler(int signal, siginfo_t *info, void* void_context)
+sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
 {
-    os_context_t *context = arch_os_get_context(&void_context);
-
     os_vm_address_t addr = arch_get_bad_addr(signal,info,context);
 
     if (addr != NULL &&
@@ -135,7 +133,7 @@ sigsegv_handler(int signal, siginfo_t *info, void* void_context)
         interrupt_handle_pending(context);
     } else if (!cheneygc_handle_wp_violation(context, addr)) {
         if(!handle_guard_page_triggered(context,addr))
-            interrupt_handle_now(signal, info, context);
+            lisp_memory_fault_error(context, addr);
     }
 }
 
