@@ -15,16 +15,22 @@
 #ifndef _SBCL_RUNTIME_H_
 #define _SBCL_RUNTIME_H_
 
-/*#define QSHOW */ /* Enable low-level debugging output? */
-
-#ifdef QSHOW
-#define FSHOW(args) fprintf args
-#define SHOW(string) FSHOW((stderr, "/%s\n", string))
+#if defined(LISP_FEATURE_SB_THREAD)
+#define thread_self() pthread_self()
+#define thread_kill pthread_kill
+#define thread_sigmask pthread_sigmask
+#define thread_mutex_lock(l) pthread_mutex_lock(l)
+#define thread_mutex_unlock(l) pthread_mutex_unlock(l)
 #else
-#define FSHOW(args)
-#define SHOW(string)
+#define thread_self() 0
+#define thread_kill kill_safely
+#define thread_sigmask sigprocmask
+#define thread_mutex_lock(l) 0
+#define thread_mutex_unlock(l) 0
 #endif
 
+/* Block blockable interrupts for each SHOW, if not 0. */
+#define QSHOW_SIGNAL_SAFE 1
 /* Enable extra-verbose low-level debugging output for signals? (You
  * probably don't want this unless you're trying to debug very early
  * cold boot on a new machine, or one where you've just messed up
@@ -34,8 +40,50 @@
  * causes output from signal handlers, and the i/o libraries aren't
  * necessarily reentrant. But it can still be very convenient for
  * figuring out what's going on when you have a signal handling
- * problem.. */
+ * problem. */
 #define QSHOW_SIGNALS 0
+/* Enable low-level debugging output, if not zero. Defaults to enabled
+ * if QSHOW_SIGNALS, disabled otherwise. Change it to 1 if you want
+ * low-level debugging output but not the whole signal mess. */
+#define QSHOW QSHOW_SIGNALS
+
+#if QSHOW
+
+#if QSHOW_SIGNAL_SAFE
+
+#include <signal.h>
+extern sigset_t blockable_sigset;
+
+#define QSHOW_BLOCK                                             \
+        sigset_t oldset;                                        \
+        thread_sigmask(SIG_BLOCK, &blockable_sigset, &oldset);
+#define QSHOW_UNBLOCK thread_sigmask(SIG_SETMASK,&oldset,0);
+#else
+#define QSHOW_BLOCK
+#define QSHOW_UNBLOCK
+#endif
+
+#ifdef LISP_FEATURE_SB_THREAD
+#define QSHOW_PREFIX fprintf(stderr, "%lu ", pthread_self());
+#else
+#define QSHOW_PREFIX
+#endif
+
+#define FSHOW(args)                                             \
+    do {                                                        \
+        QSHOW_BLOCK                                             \
+        QSHOW_PREFIX                                            \
+        fprintf args;                                           \
+        QSHOW_UNBLOCK                                           \
+    } while (0)
+#define SHOW(string) FSHOW((stderr, "/%s\n", string))
+
+#else
+
+#define FSHOW(args)
+#define SHOW(string)
+
+#endif
 
 #if QSHOW_SIGNALS
 #define FSHOW_SIGNAL FSHOW
@@ -123,6 +171,21 @@ is_lisp_pointer(lispobj obj)
     return obj & 1;
 }
 
+#include "fixnump.h"
+
+/* Is the Lisp object obj something with immediate nature (e.g. a
+ * fixnum or character or unbound marker)? */
+static inline int
+is_lisp_immediate(lispobj obj)
+{
+    return (fixnump(obj)
+            || (widetag_of(obj) == CHARACTER_WIDETAG)
+#if N_WORD_BITS == 64
+            || (widetag_of(obj) == SINGLE_FLOAT_WIDETAG)
+#endif
+            || (widetag_of(obj) == UNBOUND_MARKER_WIDETAG));
+}
+
 /* Convert from a lispobj with type bits to a native (ordinary
  * C/assembly) pointer to the beginning of the object. */
 static inline lispobj *
@@ -182,4 +245,13 @@ typedef int boolean;
 extern void *successful_malloc (size_t size);
 extern char *copied_string (char *string);
 
+#define RUNTIME_OPTIONS_MAGIC 0x31EBF355
+/* 1 for magic, 1 for boolean, 2 for struct runtime_options fields */
+#define RUNTIME_OPTIONS_WORDS (1 + 1 + 2)
+
+struct runtime_options {
+    size_t dynamic_space_size;
+    size_t thread_control_stack_size;
+};
+
 #endif /* _SBCL_RUNTIME_H_ */
