@@ -38,10 +38,36 @@
 #include "genesis/lutex.h"
 #endif
 
+/* write_runtime_options uses a simple serialization scheme that
+ * consists of one word of magic, one word indicating whether options
+ * are actually saved, and one word per struct field. */
+static void
+write_runtime_options(FILE *file, struct runtime_options *options)
+{
+    size_t optarray[RUNTIME_OPTIONS_WORDS];
+
+    memset(&optarray, 0, sizeof(optarray));
+    optarray[0] = RUNTIME_OPTIONS_MAGIC;
+
+    if (options != NULL) {
+        /* optarray[1] is a flag indicating that options are present */
+        optarray[1] = 1;
+        optarray[2] = options->dynamic_space_size;
+        optarray[3] = options->thread_control_stack_size;
+    }
+
+    if (RUNTIME_OPTIONS_WORDS !=
+        fwrite(optarray, sizeof(size_t), RUNTIME_OPTIONS_WORDS, file)) {
+        perror("Error writing runtime options to file");
+    }
+}
+
 static void
 write_lispobj(lispobj obj, FILE *file)
 {
-    fwrite(&obj, sizeof(lispobj), 1, file);
+    if (1 != fwrite(&obj, sizeof(lispobj), 1, file)) {
+        perror("Error writing to file");
+    }
 }
 
 static long
@@ -192,10 +218,11 @@ open_core_for_saving(char *filename)
 
 boolean
 save_to_filehandle(FILE *file, char *filename, lispobj init_function,
-                   boolean make_executable)
+                   boolean make_executable,
+                   boolean save_runtime_options)
 {
     struct thread *th;
-    os_vm_offset_t core_start_pos, core_end_pos, core_size;
+    os_vm_offset_t core_start_pos;
 
     /* Smash the enclosing state. (Once we do this, there's no good
      * way to go back, which is a sufficient reason that this ends up
@@ -283,12 +310,12 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
     {
         size_t size = (last_free_page*sizeof(long)+os_vm_page_size-1)
             &~(os_vm_page_size-1);
-        long *data = calloc(size, 1);
+        unsigned long *data = calloc(size, 1);
         if (data) {
             long offset;
             int i;
             for (i = 0; i < last_free_page; i++) {
-                data[i] = page_table[i].first_object_offset;
+                data[i] = page_table[i].region_start_offset;
             }
             write_lispobj(PAGE_TABLE_CORE_ENTRY_TYPE_CODE, file);
             write_lispobj(4, file);
@@ -323,12 +350,22 @@ save_to_filehandle(FILE *file, char *filename, lispobj init_function,
      * This is used to locate the start of the core when the runtime is
      * prepended to it. */
     fseek(file, 0, SEEK_END);
-    core_end_pos = ftell(file);
-    core_size = core_end_pos - core_start_pos;
 
-    fwrite(&core_size, sizeof(os_vm_offset_t), 1, file);
-    write_lispobj(CORE_MAGIC, file);
-    fclose(file);
+    /* If NULL runtime options are passed to write_runtime_options,
+     * command-line processing is performed as normal in the SBCL
+     * executable. Otherwise, the saved runtime options are used and
+     * all command-line arguments are available to Lisp in
+     * SB-EXT:*POSIX-ARGV*. */
+    write_runtime_options(file,
+                          (save_runtime_options ? runtime_options : NULL));
+
+    if (1 != fwrite(&core_start_pos, sizeof(os_vm_offset_t), 1, file)) {
+        perror("Error writing core starting position to file");
+        fclose(file);
+    } else {
+        write_lispobj(CORE_MAGIC, file);
+        fclose(file);
+    }
 
 #ifndef LISP_FEATURE_WIN32
     if (make_executable)
@@ -388,13 +425,20 @@ save_runtime_to_filehandle(FILE *output, void *runtime, size_t runtime_size)
     size_t padding;
     void *padbytes;
 
-    fwrite(runtime, 1, runtime_size, output);
+    if (runtime_size != fwrite(runtime, 1, runtime_size, output)) {
+        perror("Error saving runtime");
+        return 0;
+    }
 
     padding = (os_vm_page_size - (runtime_size % os_vm_page_size)) & ~os_vm_page_size;
     if (padding > 0) {
         padbytes = successful_malloc(padding);
         memset(padbytes, 0, padding);
-        fwrite(padbytes, 1, padding, output);
+        if (padding != fwrite(padbytes, 1, padding, output)) {
+            perror("Error saving runtime");
+            free(padbytes);
+            return 0;
+        }
         free(padbytes);
     }
 
@@ -434,7 +478,8 @@ prepare_to_save(char *filename, boolean prepend_runtime, void **runtime_bytes,
 }
 
 boolean
-save(char *filename, lispobj init_function, boolean prepend_runtime)
+save(char *filename, lispobj init_function, boolean prepend_runtime,
+     boolean save_runtime_options)
 {
     FILE *file;
     void *runtime_bytes = NULL;
@@ -447,5 +492,6 @@ save(char *filename, lispobj init_function, boolean prepend_runtime)
     if (prepend_runtime)
         save_runtime_to_filehandle(file, runtime_bytes, runtime_size);
 
-    return save_to_filehandle(file, filename, init_function, prepend_runtime);
+    return save_to_filehandle(file, filename, init_function, prepend_runtime,
+                              save_runtime_options);
 }
