@@ -28,7 +28,7 @@
 
 #if defined LISP_FEATURE_SPARC
 #define OS_VM_DEFAULT_PAGESIZE 8192
-#elif defined LISP_FEATURE_X86
+#elif defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
 #define OS_VM_DEFAULT_PAGESIZE 4096
 #else
 #error "Don't know OS_VM_DEFAULT_PAGESIZE"
@@ -197,33 +197,25 @@ boolean is_valid_lisp_addr(os_vm_address_t addr)
 #if defined LISP_FEATURE_GENCGC
 
 void
-sigsegv_handler(int signal, siginfo_t *info, void* void_context)
+sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
 {
-    os_context_t *context = arch_os_get_context(&void_context);
     void* fault_addr = (void*)info->si_addr;
 
     if (!gencgc_handle_wp_violation(fault_addr))
-         if(!handle_guard_page_triggered(context, fault_addr))
-#ifdef LISP_FEATURE_C_STACK_IS_CONTROL_STACK
-            arrange_return_to_lisp_function(context,
-                                            SymbolFunction(MEMORY_FAULT_ERROR));
-#else
-            interrupt_handle_now(signal, info, context);
-#endif
+        if(!handle_guard_page_triggered(context, fault_addr))
+            lisp_memory_fault_error(context, fault_addr);
 }
 
 #else
 
 static void
-sigsegv_handler(int signal, siginfo_t *info, void* void_context)
+sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
 {
-    os_context_t *context = arch_os_get_context(&void_context);
-    os_vm_address_t addr;
+    os_vm_address_t addr = arch_get_bad_addr(signal, info, context);
 
-    addr = arch_get_bad_addr(signal, info, context);
-    if(!interrupt_maybe_gc(signal, info, context)) {
-        if(!handle_guard_page_triggered(context,addr))
-            interrupt_handle_now(signal, info, context);
+    if (!cheneygc_handle_wp_violation(context, addr)) {
+        if (!handle_guard_page_triggered(context,addr))
+            lisp_memory_fault_error(context, fault_addr);
     }
 }
 
@@ -236,8 +228,6 @@ os_install_interrupt_handlers()
                                                  sigsegv_handler);
 
 #ifdef LISP_FEATURE_SB_THREAD
-    undoably_install_low_level_interrupt_handler(SIG_INTERRUPT_THREAD,
-                                                 interrupt_thread_handler);
     undoably_install_low_level_interrupt_handler(SIG_STOP_FOR_GC,
                                                  sig_stop_for_gc_handler);
 #endif
