@@ -1,4 +1,3 @@
-
 #ifdef LISP_FEATURE_SB_THREAD
 #include <architecture/i386/table.h>
 #include <i386/user_ldt.h>
@@ -25,11 +24,11 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#ifdef MAC_OS_X_VERSION_10_5
+#if __DARWIN_UNIX03
 #include <sys/_structs.h>
 #endif
 
-#ifdef MAC_OS_X_VERSION_10_5
+#if __DARWIN_UNIX03
 
 typedef struct __darwin_ucontext darwin_ucontext;
 typedef struct __darwin_mcontext64 darwin_mcontext;
@@ -65,9 +64,10 @@ pthread_mutex_t mach_exception_lock = PTHREAD_MUTEX_INITIALIZER;
 
 kern_return_t mach_thread_init(mach_port_t thread_exception_port);
 
-void sigill_handler(int signal, siginfo_t *siginfo, void *void_context);
-void sigtrap_handler(int signal, siginfo_t *siginfo, void *void_context);
-void memory_fault_handler(int signal, siginfo_t *siginfo, void *void_context);
+void sigill_handler(int signal, siginfo_t *siginfo, os_context_t *context);
+void sigtrap_handler(int signal, siginfo_t *siginfo, os_context_t *context);
+void memory_fault_handler(int signal, siginfo_t *siginfo,
+                          os_context_t *context);
 
 /* exc_server handles mach exception messages from the kernel and
  * calls catch exception raise. We use the system-provided
@@ -233,7 +233,7 @@ void signal_emulation_wrapper(x86_thread_state64_t *thread_state,
 
     build_fake_signal_context(context, thread_state, float_state);
 
-    block_blockable_signals();
+    block_blockable_signals(0, 0);
 
     handler(signal, siginfo, context);
 
@@ -277,17 +277,15 @@ void dump_context(x86_thread_state64_t *context)
 #endif
 
 void
-control_stack_exhausted_handler(int signal, siginfo_t *siginfo, void *void_context) {
-    os_context_t *context = arch_os_get_context(&void_context);
-
+control_stack_exhausted_handler(int signal, siginfo_t *siginfo,
+                                os_context_t *context) {
+    unblock_signals_in_context_and_maybe_warn(context);
     arrange_return_to_lisp_function
         (context, StaticSymbolFunction(CONTROL_STACK_EXHAUSTED_ERROR));
 }
 
 void
-undefined_alien_handler(int signal, siginfo_t *siginfo, void *void_context) {
-    os_context_t *context = arch_os_get_context(&void_context);
-
+undefined_alien_handler(int signal, siginfo_t *siginfo, os_context_t *context) {
     arrange_return_to_lisp_function
         (context, StaticSymbolFunction(UNDEFINED_ALIEN_VARIABLE_ERROR));
 }
@@ -354,11 +352,15 @@ catch_exception_raise(mach_port_t exception_port,
              * protection so the error handler has some headroom, protect the
              * previous page so that we can catch returns from the guard page
              * and restore it. */
-            protect_control_stack_guard_page_thread(0, th);
-            protect_control_stack_return_guard_page_thread(1, th);
+            protect_control_stack_guard_page(0, th);
+            protect_control_stack_return_guard_page(1, th);
 
             backup_thread_state = thread_state;
             open_stack_allocation(&thread_state);
+            /* Reserve a 256 byte zone for signal handlers
+             * to use on the interrupted thread stack.
+             */
+            stack_allocate(&thread_state, 256);
 
             /* Save thread state */
             target_thread_state =
@@ -392,13 +394,14 @@ catch_exception_raise(mach_port_t exception_port,
              * unprotect this one. This works even if we somehow missed
              * the return-guard-page, and hit it on our way to new
              * exhaustion instead. */
-            protect_control_stack_guard_page_thread(1, th);
-            protect_control_stack_return_guard_page_thread(0, th);
+            protect_control_stack_guard_page(1, th);
+            protect_control_stack_return_guard_page(0, th);
         }
         else if (addr >= undefined_alien_address &&
                  addr < undefined_alien_address + os_vm_page_size) {
             backup_thread_state = thread_state;
             open_stack_allocation(&thread_state);
+            stack_allocate(&thread_state, 256);
 
             /* Save thread state */
             target_thread_state =
@@ -428,6 +431,7 @@ catch_exception_raise(mach_port_t exception_port,
 
             backup_thread_state = thread_state;
             open_stack_allocation(&thread_state);
+            stack_allocate(&thread_state, 256);
 
             /* Save thread state */
             target_thread_state =
@@ -504,6 +508,7 @@ catch_exception_raise(mach_port_t exception_port,
 
             backup_thread_state = thread_state;
             open_stack_allocation(&thread_state);
+            stack_allocate(&thread_state, 256);
 
             /* Save thread state */
             target_thread_state =
@@ -574,7 +579,7 @@ mach_exception_handler(void *port)
   /* mach_msg_server should never return, but it should dispatch mach
    * exceptions to our catch_exception_raise function
    */
-  abort();
+  lose("mach_msg_server returned");
 }
 
 /* Sets up the thread that will listen for mach exceptions. note that
