@@ -165,7 +165,7 @@
               ;; the presence of the IR1 stepper instrumentation (and
               ;; is thus again failing now that the instrumentation is
               ;; no more).
-              :fails-on '(or :x86 :x86-64 :alpha :mips))
+              :fails-on '(or :x86 :x86-64 :alpha :mips :ppc))
     (assert (verify-backtrace
              (lambda () (test #'not-optimized))
              (list *undefined-function-frame*
@@ -207,8 +207,13 @@
 
 (with-test (:name (:throw :no-such-tag)
             :fails-on '(or
+                        (and :x86 :openbsd)
                         (and :x86 :sunos)
+                        (and :x86 :darwin)
+                        (and :x86 :linux)
                         (and :x86-64 :darwin)
+                        (and :x86-64 :linux)
+                        (and :x86-64 :openbsd)
                         (and :sparc :linux)
                         :alpha
                         :mips))
@@ -256,6 +261,16 @@
   `(let ((sb-debug:*show-entry-point-details* ,bool))
      ,@body))
 
+(defun bug-354 (x)
+  (error "XEPs in backtraces: ~S" x))
+
+(with-test (:name :bug-354)
+  (with-details t
+    (assert (not (verify-backtrace (lambda () (bug-354 354))
+                                   '((bug-354 &rest)
+                                     ((sb-c::tl-xep bug-354) &rest))))))
+  (assert (verify-backtrace (lambda () (bug-354 354)) '((bug-354 354)))))
+
 ;;; FIXME: This test really should be broken into smaller pieces
 (with-test (:name (:backtrace :misc)
             :fails-on '(or (and :x86 (or :sunos)) (and :x86-64 :darwin)))
