@@ -55,12 +55,12 @@
 ;;;; Build the tool again, this time as a shared object, and load it
 
 (run "cc" "stack-alignment-offset.c"
-     #+(and (or linux freebsd) (or x86-64 ppc mips)) "-fPIC"
+     #+(and (not darwin) (or x86-64 ppc mips)) "-fPIC"
      #+(and x86-64 darwin) "-arch" #+(and x86-64 darwin) "x86_64"
      #+darwin "-bundle" #-darwin "-shared"
      "-o" "stack-alignment-offset.so")
 
-(load-shared-object "stack-alignment-offset.so")
+(load-shared-object (truename "stack-alignment-offset.so"))
 
 (define-alien-routine stack-alignment-offset int (alignment int))
 (define-alien-routine trampoline int (callback (function int)))
